--!parent
--!encoding=utf-8

--****************************************************************************
-- @file LSW CORE LIB: CONSOLE
-- @author Valentin Schmidt
-- @version	0.2
--****************************************************************************

property $
property ancestor
property _regex

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
	$.loadXtras(me.libdir, ["Console.x32"], [])
	me.ancestor = xtra("Console").new()
	me._regex = $.include(me.LIBDIR&"regex.js")
	me._loadConstantsColor()
	me._loadConstantsSW()
end

----------------------------------------
--
----------------------------------------
on __destruct (me)
	if not voidP(me.ancestor) then
		me.consoleClose()
		me.ancestor = VOID
	end if
end

----------------------------------------
-- Opens a non-interactive console. i.e. for printing output only
-- @return {bool} success
----------------------------------------
on show (me, set_focus, buf_size, hidden, caption)
	if voidP(buf_size) then buf_size = 500
	if voidP(hidden) then hidden = FALSE
	if voidP(caption) then caption = "Console"
			
	-- overwrites 'put' etc.
	$.include(me.LIBDIR&"handlers.ls")

	-- attach to existing or create new console
	if me.consoleExists() then
		ok = me.consoleAttach(set_focus)
	else
		ok = me.consoleCreate(set_focus, buf_size, hidden)
	end if

	if not ok then return FALSE

	me.consoleSetTitle(caption)

	stdinCol  = me.FOREGROUND_RED + me.FOREGROUND_GREEN + me.FOREGROUND_BLUE --  7 = White
	stdoutCol = me.FOREGROUND_GREEN + me.FOREGROUND_INTENSITY -- 10 = Light Green
	stderrCol = me.FOREGROUND_RED + me.FOREGROUND_INTENSITY
	me.consoleSetColors(stdinCol, stdoutCol, stderrCol)

	return TRUE
end

----------------------------------------
-- --, replBreaks
----------------------------------------
on _enc (me, arg)
	case ilk(arg) of
		#void: return "<Void>"
		#symbol: return "#"&arg
--		#string:
			--arg = me._regex.fixEOL(arg)
--			return QUOTE&arg&QUOTE
		otherwise:
			return string(arg)
	end case
end

----------------------------------------
-- Console color constants
----------------------------------------
on _loadConstantsColor (me)
	me[#FOREGROUND_BLUE] = 1
	me[#FOREGROUND_GREEN] = 2
	me[#FOREGROUND_RED] = 4
	me[#FOREGROUND_INTENSITY] = 8 -- Foreground color is intensified.
	me[#BACKGROUND_BLUE] = 16
	me[#BACKGROUND_GREEN] = 32
	me[#BACKGROUND_RED] = 64
	me[#BACKGROUND_INTENSITY] = 128 -- Background color is intensified.
end

----------------------------------------
-- ShowWindow constants
----------------------------------------
on _loadConstantsSW (me)
	me[#SW_HIDE] = 0
	me[#SW_SHOWNORMAL] = 1
	me[#SW_SHOWMINIMIZED] = 2
	me[#SW_MAXIMIZE] = 3
	me[#SW_SHOWMAXIMIZED] = 3
	me[#SW_SHOWNOACTIVATE] = 4
	me[#SW_SHOW] = 5
	me[#SW_MINIMIZE] = 6
	me[#SW_SHOWMINNOACTIVE] = 7
	me[#SW_SHOWNA] = 8
	me[#SW_RESTORE] = 9
	me[#SW_SHOWDEFAULT] = 10
	me[#SW_FORCEMINIMIZE] = 11
end
