--!parent
--!encoding=utf-8

--****************************************************************************
-- @file LSW CORE LIB: JSON
-- @author Valentin Schmidt
-- @version 0.3
--****************************************************************************

property $
property _js_helper

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
	me._js_helper = $.include(me.libdir&"json_helper.js") -- JS helper functions
	return me
end

----------------------------------------
-- JSON_DECODE
-- @param {string|byteArray} data
-- @param {bool} [symbol_props=FALSE]
-- @return {any}
----------------------------------------
on decode (me, data, symbol_props)
	if ilk(data)=#byteArray then
		data.position = 1
		data = data.readRawString(data.length)
	end if
	res = me._js_helper.json_decode(data, symbol_props)
	-- TMP
	--if ilk(res)<>#propList then $.file.putString($.PATH&"debug.json", data)
	return res
end

----------------------------------------
-- JSON_ENCODE
-- @param {any} c
-- @return {any}
----------------------------------------
on encode (me, o)
	case ilk(o) of
		#void:
			return "null"
		#integer, #float:
			return string(o)
		#string:
			o = me._replace("\","\\",o)
			return QUOTE & _js_helper.json_escape_string(o) & QUOTE
		#list:
			res = []
			repeat with v in o
				res.add(me.encode(v))
			end repeat
			return "[" & me._implode(",", res) & "]"
		#propList:
			res = []
			cnt = count(o)
			repeat with i = 1 to cnt
				p = o.getPropAt(i)
				v = o[i]
				res.add( me.encode(p)&":"&me.encode(v) )
			end repeat
			return "{" & me._implode(",", res) & "}"
		#symbol:
			--return QUOTE&"_sym_"&string(o)&QUOTE
			o = me._replace("\","\\",o)
			return QUOTE & _js_helper.json_escape_string(string(o)) & QUOTE

		otherwise:
			-- put "ERROR: unsupported data type!"
			return QUOTE&"<UNSUPPORTED DATA TYPE: "&ilk(o)&">"&QUOTE

	end case
end

----------------------------------------
--
----------------------------------------
on _implode (me, delim, l)
	str=""
	repeat with i=1 to l.count
		put l[i]&delim after str
	end repeat
	delete char (str.length-delim.length+1) to str.length of str
	return str
end

----------------------------------------
--
----------------------------------------
on _replace (me, str_find, str_insert, input)
	output = ""
	findLen = str_find.length - 1
	repeat while true
		currOffset = offset(str_find, input)
		if currOffset=0 then exit repeat
		put input.char [1..currOffset] after output
		delete the last char of output
		put str_insert after output
		delete input.char [1.. (currOffset + findLen)]
	end repeat
	put input after output
	return output
end
