--!parent

--****************************************************************************
-- @file			LSW CORE LIB: QTWIDGETS - QT-EVENTS EXTENSION
-- @author   	Valentin Schmidt
-- @version  	0.2
-- @desc      Adds event constants to namespace Qt.Event (e.g. $.Qt.Event.Timer)
-- @todo
--****************************************************************************

----------------------------------------
-- @constructor
----------------------------------------
on new (me)

	evt = [:]
	
	evt[#None] = 0                               -- invalid event
	evt[#Timer] = 1                              -- timer event
	evt[#MouseButtonPress] = 2                   -- mouse button pressed
	evt[#MouseButtonRelease] = 3                 -- mouse button released
	evt[#MouseButtonDblClick] = 4                -- mouse button double click
	evt[#MouseMove] = 5                          -- mouse move
	evt[#KeyPress] = 6                           -- key pressed
	evt[#KeyRelease] = 7                         -- key released
	evt[#FocusIn] = 8                            -- keyboard focus received
	evt[#FocusOut] = 9                           -- keyboard focus lost
	evt[#Enter] = 10                             -- mouse enters widget
	evt[#Leave] = 11                             -- mouse leaves widget
	evt[#Paint] = 12                             -- paint widget
	evt[#Move] = 13                              -- move widget
	evt[#Resize] = 14                            -- resize widget
	evt[#Create] = 15                            -- after widget creation
	evt[#Destroy] = 16                           -- during widget destruction
	evt[#Show] = 17                              -- widget is shown
	evt[#Hide] = 18                              -- widget is hidden
	evt[#Close] = 19                             -- request to close widget
	evt[#Quit] = 20                              -- request to quit application
	evt[#ParentChange] = 21                      -- widget has been reparented
	evt[#ParentAboutToChange] = 131              -- sent just before the parent change is done
	--#ifdef QT3_SUPPORT
	--evt[#Reparent] = ParentChange
	--#endif
	evt[#ThreadChange] = 22                      -- object has changed threads
	evt[#WindowActivate] = 24                    -- window was activated
	evt[#WindowDeactivate] = 25                  -- window was deactivated
	evt[#ShowToParent] = 26                      -- widget is shown to parent
	evt[#HideToParent] = 27                      -- widget is hidden to parent
	evt[#Wheel] = 31                             -- wheel event
	evt[#WindowTitleChange] = 33                 -- window title changed
	evt[#WindowIconChange] = 34                  -- icon changed
	evt[#ApplicationWindowIconChange] = 35       -- application icon changed
	evt[#ApplicationFontChange] = 36             -- application font changed
	evt[#ApplicationLayoutDirectionChange] = 37  -- application layout direction changed
	evt[#ApplicationPaletteChange] = 38          -- application palette changed
	evt[#PaletteChange] = 39                     -- widget palette changed
	evt[#Clipboard] = 40                         -- internal clipboard event
	evt[#Speech] = 42                            -- reserved for speech input
	evt[#MetaCall] =  43                         -- meta call event
	evt[#SockAct] = 50                           -- socket activation
	evt[#WinEventAct] = 132                      -- win event activation
	evt[#DeferredDelete] = 52                    -- deferred delete event
	evt[#DragEnter] = 60                         -- drag moves into widget
	evt[#DragMove] = 61                          -- drag moves in widget
	evt[#DragLeave] = 62                         -- drag leaves or is cancelled
	evt[#Drop] = 63                              -- actual drop
	evt[#DragResponse] = 64                      -- drag accepted/rejected
	evt[#ChildAdded] = 68                        -- new child widget
	evt[#ChildPolished] = 69                     -- polished child widget
	--#ifdef QT3_SUPPORT
	--evt[#ChildInsertedRequest] = 67            -- send ChildInserted compatibility events to receiver
	--evt[#ChildInserted] = 70                   -- compatibility child inserted
	--evt[#LayoutHint] = 72                      -- compatibility relayout request
	--#endif
	evt[#ChildRemoved] = 71                      -- deleted child widget
	evt[#ShowWindowRequest] = 73                 -- widget's window should be mapped
	evt[#PolishRequest] = 74                     -- widget should be polished
	evt[#Polish] = 75                            -- widget is polished
	evt[#LayoutRequest] = 76                     -- widget should be relayouted
	evt[#UpdateRequest] = 77                     -- widget should be repainted
	evt[#UpdateLater] = 78                       -- request update() later
	evt[#EmbeddingControl] = 79                  -- ActiveX embedding
	evt[#ActivateControl] = 80                   -- ActiveX activation
	evt[#DeactivateControl] = 81                 -- ActiveX deactivation
	evt[#ContextMenu] = 82                       -- context popup menu
	evt[#InputMethod] = 83                       -- input method
	evt[#AccessibilityPrepare] = 86              -- accessibility information is requested
	evt[#TabletMove] = 87                        -- Wacom tablet event
	evt[#LocaleChange] = 88                      -- the system locale changed
	evt[#LanguageChange] = 89                    -- the application language changed
	evt[#LayoutDirectionChange] = 90             -- the layout direction changed
	evt[#Style] = 91                             -- internal style event
	evt[#TabletPress] = 92                       -- tablet press
	evt[#TabletRelease] = 93                     -- tablet release
	evt[#OkRequest] = 94                         -- CE (Ok) button pressed
	evt[#HelpRequest] = 95                       -- CE (?)  button pressed
	evt[#IconDrag] = 96                          -- proxy icon dragged
	evt[#FontChange] = 97                        -- font has changed
	evt[#EnabledChange] = 98                     -- enabled state has changed
	evt[#ActivationChange] = 99                  -- window activation has changed
	evt[#StyleChange] = 100                      -- style has changed
	evt[#IconTextChange] = 101                   -- icon text has changed
	evt[#ModifiedChange] = 102                   -- modified state has changed
	evt[#MouseTrackingChange] = 109              -- mouse tracking state has changed
	evt[#WindowBlocked] = 103                    -- window is about to be blocked modally
	evt[#WindowUnblocked] = 104                  -- windows modal blocking has ended
	evt[#WindowStateChange] = 105
	evt[#ToolTip] = 110
	evt[#WhatsThis] = 111
	evt[#StatusTip] = 112
	evt[#ActionChanged] = 113
	evt[#ActionAdded] = 114
	evt[#ActionRemoved] = 115
	evt[#FileOpen] = 116                         -- file open request
	evt[#Shortcut] = 117                         -- shortcut triggered
	evt[#ShortcutOverride] = 51                  -- shortcut override request
	--#ifdef QT3_SUPPORT
	--evt[#Accel] = 30                           -- accelerator event
	--evt[#AccelAvailable] = 32                  -- accelerator available event
	--evt[#AccelOverride] = ShortcutOverride     -- accelerator override event
	--#endif
	evt[#WhatsThisClicked] = 118
	--#ifdef QT3_SUPPORT
	--evt[#CaptionChange] = WindowTitleChange
	--evt[#IconChange] = WindowIconChange
	--#endif
	evt[#ToolBarChange] = 120                    -- toolbar visibility toggled
	evt[#ApplicationActivate] = 121              -- application has been changed to active
	--evt[#ApplicationActivated] = ApplicationActivate -- deprecated
	evt[#ApplicationDeactivate] = 122            -- application has been changed to inactive
	--evt[#ApplicationDeactivated] = ApplicationDeactivate -- deprecated
	evt[#QueryWhatsThis] = 123                   -- query what's this widget help
	evt[#EnterWhatsThisMode] = 124
	evt[#LeaveWhatsThisMode] = 125
	evt[#ZOrderChange] = 126                     -- child widget has had its z-order changed
	evt[#HoverEnter] = 127                       -- mouse cursor enters a hover widget
	evt[#HoverLeave] = 128                       -- mouse cursor leaves a hover widget
	evt[#HoverMove] = 129                        -- mouse cursor move inside a hover widget
	evt[#AccessibilityHelp] = 119                -- accessibility help text request
	evt[#AccessibilityDescription] = 130         -- accessibility description text request
	--#ifdef QT_KEYPAD_NAVIGATION
	--evt[#EnterEditFocus] = 150                 -- enter edit mode in keypad navigation
	--evt[#LeaveEditFocus] = 151                 -- enter edit mode in keypad navigation
	--#endif
	evt[#AcceptDropsChange] = 152
	evt[#MenubarUpdated] = 153                   -- Support event for Q3MainWindow which needs to knwow when QMenubar is updated.
	evt[#ZeroTimerEvent] = 154                   -- Used for Windows Zero timer events
	evt[#GraphicsSceneMouseMove] = 155           -- GraphicsView
	evt[#GraphicsSceneMousePress] = 156
	evt[#GraphicsSceneMouseRelease] = 157
	evt[#GraphicsSceneMouseDoubleClick] = 158
	evt[#GraphicsSceneContextMenu] = 159
	evt[#GraphicsSceneHoverEnter] = 160
	evt[#GraphicsSceneHoverMove] = 161
	evt[#GraphicsSceneHoverLeave] = 162
	evt[#GraphicsSceneHelp] = 163
	evt[#GraphicsSceneDragEnter] = 164
	evt[#GraphicsSceneDragMove] = 165
	evt[#GraphicsSceneDragLeave] = 166
	evt[#GraphicsSceneDrop] = 167
	evt[#GraphicsSceneWheel] = 168
	evt[#KeyboardLayoutChange] = 169             -- keyboard layout changed
	evt[#DynamicPropertyChange] = 170            -- A dynamic property was changed through setProperty/property
	evt[#TabletEnterProximity] = 171
	evt[#TabletLeaveProximity] = 172
	evt[#NonClientAreaMouseMove] = 173
	evt[#NonClientAreaMouseButtonPress] = 174
	evt[#NonClientAreaMouseButtonRelease] = 175
	evt[#NonClientAreaMouseButtonDblClick] = 176
	evt[#MacSizeChange] = 177                    -- when the Qt::WA_Mac{NormalSmallMini}Size changes
	evt[#ContentsRectChange] = 178               -- sent by QWidget::setContentsMargins (internal)
	evt[#MacGLWindowChange] = 179                -- Internal! the window of the GLWidget has changed
	evt[#FutureCallOut] = 180
	evt[#GraphicsSceneResize ] = 181
	evt[#GraphicsSceneMove ] = 182
	evt[#CursorChange] = 183
	evt[#ToolTipChange] = 184
	evt[#NetworkReplyUpdated] = 185              -- Internal for QNetworkReply
	evt[#GrabMouse] = 186
	evt[#UngrabMouse] = 187
	evt[#GrabKeyboard] = 188
	evt[#UngrabKeyboard] = 189
	evt[#MacGLClearDrawable] = 191               -- Internal Cocoa the window has changed so we must clear
	evt[#StateMachineSignal] = 192
	evt[#StateMachineWrapped] = 193
	evt[#TouchBegin] = 194
	evt[#TouchUpdate] = 195
	evt[#TouchEnd] = 196
	--#ifndef QT_NO_GESTURES
	--evt[#NativeGesture] = 197                  -- Internal for platform gesture support
	--#endif
	evt[#RequestSoftwareInputPanel] = 199
	evt[#CloseSoftwareInputPanel] = 200
	evt[#UpdateSoftKeys] = 201                   -- Internal for compressing soft key updates
	evt[#WinIdChange] = 203
	--#ifndef QT_NO_GESTURES
	--evt[#Gesture] = 198
	--evt[#GestureOverride] = 202
	--#endif
	
	me[#Event] = evt
	return me
end