--!parent
--!encoding=utf-8 äöü

--****************************************************************************
-- @file LSW CORE LIB: QTWIDGETS (OOP API VERSION)
-- @author Valentin Schmidt
-- @version 0.4
-- @requires Xtra("QtWidgets")
--****************************************************************************

property $

property _qx

property _dropHandler
property _dropTarget
property _dropMap
property _signalClass

----------------------------------------
-- @constructor
----------------------------------------
on new (me, flavor)

	-- check/load xtra
	if voidP(flavor) then flavor = "minimal"
	$.loadXtras(me.libdir&"Flavors"&$.PD&flavor&$.PD, ["QtWidgets.x32"], ["QtWidgets.xtra"])

	-- libs
	$.import("detacher")
	$.import("file")
	$.import("filesystem")

	--x = #qt

	-- load xtra
	me._qx = xtra("QtWidgets").new()

	-- autoload all widget/dialog/object classes
	me._autoloadClasses()

	-- load extensions
	$.inherit(me, $.include(me.libdir&"constants.ls"))
	$.inherit(me, $.include(me.libdir&"events.ls"))
	if flavor="scintilla" then
		$.inherit(me, $.include(me.libdir&"sci_constants.ls"))
	end if

	-- new
	me._dropMap = [:]

	me._signalClass = $.include(me.libDir&"classes\Signal.ls")

	return me
end

----------------------------------------
-- Sets drop callback for widget
-- @param {symbol} dropHandler
-- @param {object} [dropTarget=_movie]
----------------------------------------
on registerDropHandler (me, srcWidget, dropHandler, dropTarget)
	me._dropMap.addProp(srcWidget._id, [dropHandler, dropTarget, srcWidget])
	me._qx.setDropCallback(#_detachDrop, me)
end

----------------------------------------
-- Checks if 2 Qt objects are equal
-- @param {instance|int} obj1
-- @param {instance|int} obj2
-- @return {bool}
----------------------------------------
on equal (me, obj1, obj2)
	if ilk(obj1)=#instance then obj1 = obj1._id
	if ilk(obj2)=#instance then obj2 = obj2._id
	return (obj1=obj2)
end

----------------------------------------
--
----------------------------------------
on _autoloadClasses (me)
	fslib = $.filesystem

	od = _player.itemdelimiter
	_player.itemdelimiter = "."

	-- load dialog classes
	p = $.sanitizePath(me.libdir&"classes/dialog/")
	filenames = fslib.folderList(p)
	repeat with f in filenames
		if the last item of f<>"ls" then next repeat
		me[symbol(f.item[1])] = $.include(p&f)
	end repeat

	-- load object classes
	p = $.sanitizePath(me.libdir&"classes/object/")
	filenames = fslib.folderList(p)
	repeat with f in filenames
		if the last item of f<>"ls" then next repeat
		me[symbol(f.item[1])] = $.include(p&f)
	end repeat

	-- load widget classes
	p = $.sanitizePath(me.libdir&"classes/widget/")
	filenames = fslib.folderList(p)
	repeat with f in filenames
		if the last item of f<>"ls" then next repeat
		me[symbol(f.item[1])] = $.include(p&f)
	end repeat

	_player.itemdelimiter = od
end

----------------------------------------
-- Destructor called by LSW
-- @callback
----------------------------------------
on __destruct (me)
	me._qx = 0
end

----------------------------------------
-- Used for "detaching" drop action xtra callbacks from original thread,
-- so scripting errors in callbacks don't fail silently
-- @callback
----------------------------------------
on _detachDrop (me, dropData, dropPoint, dropTargetID, dropSourceID)
	l = me._dropMap.getProp(dropTargetID)
	if listP(l) then
		if dropSourceID then
			-- create wrapper instance
			cn = $.Qt._qx.callWidget(dropSourceID, #className)
			if cn starts "My" then delete char 1 to 2 of cn
			else delete char 1 of cn
			class = value("$.Qt."&cn)
			if ilk(class)=#script then
				dropSourceID = class._make(dropSourceID)
				--me._children.add(obj)
				--return obj
			end if
		end if
		args = [dropData, dropPoint, l[3], dropSourceID]
		$.detacher.detach(l[1], l[2], args)
	end if
end

----------------------------------------
-- low level QtWidgets xtra access
----------------------------------------
on callQt (me, handler, arg)
	if voidP(arg) then return call(handler, me._qx)
	else return call(handler, me._qx, arg)
end
