--!parent
--!encoding=utf-8

--****************************************************************************
-- @file      LSW CORE LIB: XMLPARSER
-- @author    Valentin Schmidt
-- @version   0.2
--****************************************************************************

-- xtra XmlParser -- version 11.5.0.r593
-- new object me
-- -- XmlParser Xtra --
-- parseString object me, string data -- parses the buffer
-- parseURL object me, string url, * -- parses the specified url
-- doneParsing object me -- returns true if it's done parsing a URL
-- ignoreWhitespace object me, int ignore -- toggles whether to ignore whitespace
-- getError object me -- returns the error string (if any) generated when parsing
-- makeList object me -- make a lingo list based on the XML document
-- makePropList object me -- make a lingo property list based on the XML document
-- parseByteArray object me, ByteArray byteArrayXML -- parses the byteArray created from a XML file"

property $
property ancestor

property attribProp

global n
global xmlStr

----------------------------------------
-- @constructor
----------------------------------------
on new (me)

  $.import("file")

  $.loadXtras(me.libdir, ["XmlParser.x32"], ["XmlParser PPC Xtra.xtra"])

  me.ancestor = xtra("XmlParser").new()

  return me
end

----------------------------------------
-- Read and parse local XML file
-- @param {string} fn
-- @return {bool} fileFound
----------------------------------------
on parseFile (me, fn)
  ba = me.$.file.getBytes(fn)
  if ilk(ba)<>#bytearray then return false
  me.parseByteArray(ba)
  return true
end

----------------------------------------
-- xmlParser xtra is quite nazi concerning illegal characters in ISO-8859-1.
-- To also allow characters in the range 80-9F (like e.g. the TM-symbol),
-- we change ISO-8859-1 to Windows-1252 (=a superset of ISO-8859-1!) in the XML's declaration.
-- @param {bytearray} baXmlData
-- @return {bytearray}
----------------------------------------
on fixDeclaration (me, baXmlData)
  -- find position of first ">" = numtochar(62)
  repeat with i = 1 to 256
    if baXmlData[i]=62 then exit repeat
  end repeat
  if i>=256 then return baXmlData
  baXmlData.position = 1
  declaration = baXmlData.readRawString(i)
  if not (declaration contains "ISO-8859-1") then return baXmlData
  declaration = "<?xml version="&QUOTE&"1.0"&QUOTE&" encoding="&QUOTE&"Windows-1252"&QUOTE&"?>"
  baNew = bytearray(declaration)
  baNew.position = baNew.length+1
  baXmlData.position = i+1
  baNew.writeByteArray(baXmlData.readByteArray(baXmlData.length-i-1))
  return baNew
end if

----------------------------------------
--
----------------------------------------
on makeSimplePropList (me, pl, symbolProps)

  me.attribProp = "__attributes"
  if symbolProps then me.attribProp = symbol(me.attribProp)

  data = [:]
  me._parseNode (data, pl, symbolProps)
  return data
end

----------------------------------------
--
----------------------------------------
on streamParseBytes (me, ba, parseLevel, rowCallbackHandler, rowCallbackTarget)
  if voidP(rowCallbackTarget) then rowCallbackTarget = _movie
	
	fileLen = ba.length
	
	startPos = 1
	
  -- check for BOM 0xef, 0xbb, 0xbf
  if ba[1] = 239 then startPos = 4

  -- check for declaration (<?xml ... ?>)
  len = min(fileLen, 128)
  tmp = ba.readRawString(len)
  pos = offset("?>", tmp)
  if pos then startPos=startPos+pos+1
	
  first = 1
  stack = []
  lastChar = 0
  elemStart = 0
  isEndElem = FALSE

  repeat with i = startPos to fileLen
    c = ba[i]

    if c=60 then -- <
      elemStart = i
      isEndElem = FALSE

    else if c=47 and lastChar=60 then -- </ 3c 2f
      isEndElem = TRUE

    else if c=62 then -- >

      ba.position = elemStart+1
      elem = ba.readRawString(i-elemStart-1)

      if isEndElem then
        lastPos = stack.getLast()
        stack.deleteAt(stack.count)
        --put "close", stack.count

        if parseLevel=stack.count then
          ba.position = lastPos
          xmlStr = ba.readRawString(i-lastPos+1)

          me.parseString(xmlStr)
          -- parseByteArray object me, ByteArray byteArrayXML -- parses the byteArray created from a XML file"

          val = me.makePropList()
          val = me.makeSimplePropList(val)

          ok = call(rowCallbackHandler, rowCallbackTarget, val)
          if ok=FALSE then return
        end if

      else
        --put "open", stack.count
        stack.add(elemStart)
      end if

    end if

    lastChar = c
  end repeat
end

----------------------------------------
-- unbuffered
----------------------------------------
on streamParseFile (me, fn, parseLevel, rowCallbackHandler, rowCallbackTarget)
  if voidP(rowCallbackTarget) then rowCallbackTarget = _movie

  $.import("file")

  fp = $.file.fopen(fn, "r")
  fileLen = $.file.fsize(fp)

  -- check for BOM 0xef, 0xbb, 0xbf
  tmp = $.file.freadbytes(fp, 3)
  if tmp[1] = 239 then
    fileLen = fileLen-3
  else
    $.file.fseek(fp, 0)
  end if

  -- check for declaration <?xml ... ?>
  len = min(fileLen, 128)
  tmp = $.file.freadbytes(fp, len)
  tmp = tmp.readRawString(tmp.length)
  pos = offset("?>", tmp)
  if pos then pos=pos+1
  $.file.fseek(fp, -len+pos, 1)
  fileLen = fileLen-pos

  first = 1
  stack = []
  lastChar = 0
  elemStart = 0
  isEndElem = FALSE

  repeat with i = 1 to fileLen
    c = $.file.freadint8(fp) --ba[i]

    if c=60 then -- <
      elemStart = i
      isEndElem = FALSE

    else if c=47 and lastChar=60 then -- </ 3c 2f
      isEndElem = TRUE

    else if c=62 then -- >

			--ba.position = elemStart+1
      $.file.fseek(fp, elemStart+1)
			--elem = ba.readRawString(i-elemStart-1)
      elem = $.file.freadstring(fp, i-elemStart-1)

      if isEndElem then
        lastPos = stack.getLast()
        stack.deleteAt(stack.count)
        --put "close", stack.count

        if parseLevel=stack.count then
          --ba.position = lastPos
          $.file.fseek(fp, lastPos)
          --xmlStr = ba.readRawString(i-lastPos+1)
          xmlStr = $.file.freadstring(fp, i-lastPos+1)

          me.parseString(xmlStr)
          -- parseByteArray object me, ByteArray byteArrayXML -- parses the byteArray created from a XML file"

          val = me.makePropList()
          val = me.makeSimplePropList(val)

          ok = call(rowCallbackHandler, rowCallbackTarget, val)
          if ok=FALSE then return
        end if

      else
        --put "open", stack.count
        stack.add(elemStart)
      end if

    end if

    lastChar = c
  end repeat
end

----------------------------------------
-- buffered
----------------------------------------
on streamParseFileBuffered (me, fn, parseLevel, rowCallbackHandler, rowCallbackTarget)
  if voidP(rowCallbackTarget) then rowCallbackTarget = _movie

  $.import("file")

  fp = $.file.fopen(fn, "r")
  fileLen = $.file.fsize(fp)

  -- check for BOM 0xef, 0xbb, 0xbf
  tmp = $.file.freadbytes(fp, 3)
  if tmp[1] = 239 then
    fileLen = fileLen-3
  else
    $.file.fseek(fp, 0)
  end if

  -- check for declaration <?xml ... ?>
  len = min(fileLen, 128)
  tmp = $.file.freadbytes(fp, len)
  tmp = tmp.readRawString(tmp.length)
  pos = offset("?>", tmp)
  if pos then pos=pos+1
  $.file.fseek(fp, -len+pos, 1)
  fileLen = fileLen-pos

  first = 1
  stack = []
  lastChar = 0
  elemStart = 0
  isEndElem = FALSE
  
	bufSize = 16384
	cnt = fileLen/bufSize

	-- ceil
--  floor = bitOr( n, 0 )
--  if( floor >= n ) then return floor
--  else return floor + 1

  lastBufSize = fileLen mod bufSize
  if lastBufSize>0 then cnt = cnt+1


	restStr = ""
	restBa = bytearray()


	-- buffer loop
	repeat with i = 1 to cnt
		
		-- read bufSize bytes into byteArray
		if i<cnt then
			len = bufSize
		else
			len = lastBufSize
		end if
		
		-- byte loop
		ba = $.file.freadbytes(fp, len)
	  repeat with i = 1 to len
	    c = ba[i]
	
	    if c=60 then -- "<"
	      elemStart = i
	      isEndElem = FALSE
	
	    else if c=47 and lastChar=60 then -- "</" 3c 2f
	      isEndElem = TRUE
	
	    else if c=62 then -- ">"
	
	      ba.position = elemStart+1
	      elem = ba.readRawString(i-elemStart-1)
	
	      if isEndElem then
	        lastPos = stack.getLast()
	        stack.deleteAt(stack.count)
	        --put "close", stack.count
	
	        if parseLevel=stack.count then
	          ba.position = lastPos
	          
	          --xmlStr = ba.readRawString(i-lastPos+1)
	          --put xmlStr after restStr
	          --me.parseString(xmlStr)
	          
	          xmlBa = ba.readByteArray(i-lastPos+1)
	          me.parseByteArray(xmlBa, restBa)
	
	          val = me.makeSimplePropList(me.makePropList())
	
	          ok = call(rowCallbackHandler, rowCallbackTarget, val)
	          if ok=FALSE then return
	        end if
	
	      else
	      	
	        --put "open", stack.count
	        stack.add(elemStart)
	        
	      end if
	
	    end if
	
	    lastChar = c
	    
	  end repeat -- byte loop

		-- get rest
		
		
	end repeat -- buffer loop

end

----------------------------------------
-- handle attributes?
----------------------------------------
on _parseNode (me, data, node, symbolProps)

  -- either chardata or children
  if not voidP(node["chardata"]) then
    val = node.chardata
  else
  	
		cntChild = value("node.child.count")
		--put "cntChild", cntChild, cntChild=0
		if cntChild=0 then
			val=""
		else
    	val = [:]
	    if not voidP(node["attributes"]) then
	      cnt = node["attributes"].count
	      if cnt then
	
	        val[me.attribProp] = [:]
	        repeat with i = 1 to cnt
	          prop = node["attributes"].getPropAt(i)
	          if symbolProps then prop = symbol(prop)
	          val[me.attribProp][prop] = node["attributes"][i]
	        end repeat
	      end if
	    end if
	
	    --cnt = value("node.child.count")
	    if not voidP(cntChild) then
	
	      repeat with i = 1 to cntChild
	        me._parseNode(val, node.child[i], symbolProps)
	      end repeat
	
	    end if
	
		end if
	
  end if

  prop = value("node.name")
  if not voidP(prop) then

    if symbolProps then prop = symbol(prop)
    if voidP(data[prop]) then
      data.addProp(prop, val)
    else
      if ilk(data[prop])<>#list then
        data[prop] = [data[prop]]
      end if
      data[prop].add(val)
    end if

  end if
end
