--!parent
--!encoding=utf-8

--****************************************************************************
-- @file MyFlickrManager main class
-- @author Valentin Schmidt
-- @version 0.12
--****************************************************************************

global $

property ancestor

property _account_settings

property _current_user_id
property _current_set_id
property _current_photo

property _dummy_icon
property _qimg

property _cache_dir
property _downloads_dir

property _download_queue
property _download_queue_len

property _upload_queue

----------------------------------------
--
----------------------------------------
on new (me)

	-- config
	APP_KEY = "448e985901322f7a9a7d5f7b0315023e"
	APP_SECRET = "ce50c82c8f34382a"

	-- load libs
	$.import("console").show(FALSE, 2000, TRUE, "Network Console")
	$.import("filesystem")
	$.import("flickr", [APP_KEY, APP_SECRET])
	$.import("qt")
	$.import("registry")

	-- register special protocol "flickrauth:", listen for WM_COPYDATA messages
	me.registerProtocol()
	$.msg.msg_listen([$.msg.WM_COPYDATA], VOID, #slotAuthMessageReceived, me)

	$.Qt.Resource.registerResourceFile($.PATH&"resources\main.rcc")

	me._dummy_icon = image(200,200,32)
	me._dummy_icon.fill(me._dummy_icon.rect, rgb(128,128,128))

	me._qimg = $.Qt.Image.new()

	me._cache_dir = $.PATH&"data\cache"
	if not $.filesystem.folderExists(me._cache_dir) then
		$.filesystem.folderCreate(me._cache_dir)
	end if

	me._downloads_dir = $.PATH&"data\downloads"
	if not $.filesystem.folderExists(me._downloads_dir) then
		$.filesystem.folderCreate(me._downloads_dir)
	end if

	-- create main window
	uiLoader = $.Qt.UiLoader.new()
	me.ancestor = uiLoader.load($.PATH&"resources\main.ui")
	me.connect($.Qt.Event.Close, _movie, #halt)

	me.setProp(#progressBar, $.Qt.ProgressBar.new())
	me.progressBar.setValue(0)
	me.progressBar.setTextVisible(FALSE)
	me.progressBar.setMaximumSize(200, 16)
	me.statusbar.addPermanentWidget(me.progressBar)

	me.actionShowNetworkConsole.connect(#toggled, me, #slotShowConsoleToggled)
	me.actionAbout.connect(#triggered, me, #slotAbout)

	----------------------------------------
	-- Tab 0: Accounts
	----------------------------------------
	me._account_settings = $.Qt.Settings.new($.Qt.QSettings.IniFormat, $.PATH&"data\accounts.ini")
	repeat with account_name in me._account_settings.childGroups()
		listItem = $.Qt.ListWidgetItem.new()
		listItem.setText(account_name)

		me._account_settings.beginGroup(account_name)
		listItem.setData($.Qt.UserRole, me._account_settings.value("access_token"))
		listItem.setData($.Qt.UserRole+1, me._account_settings.value("access_token_secret"))
		me._account_settings.endGroup()

		me.listWidgetAccounts.addItem(listItem)
	end repeat

	me.listWidgetAccounts.connect(#doubleClicked, me, #slotAccountDoubleClicked)
	me.listWidgetAccounts.connect(#customContextMenuRequested, me, #slotAccountContextMenuRequested)

	me.pushButtonNew.connect(#clicked, me, #slotAccountNew)

	----------------------------------------
	-- Tab 1: Photo Sets
	----------------------------------------
	me.listWidgetPhotoSets.connect(#doubleClicked, me, #slotPhotoSetDoubleClicked)
	me.listWidgetPhotoSets.setRawDropMode(TRUE)
	$.Qt.registerDropHandler(me.listWidgetPhotoSets, #slotPhotoSetDropped)
	me.listWidgetPhotoSets.connect(#customContextMenuRequested, me, #slotPhotoSetContextMenuRequested)
	me.tabWidget.setTabEnabled(1, FALSE)

	----------------------------------------
	-- Tab 2: Photos
	----------------------------------------
	me.setProp(#listWidgetPhotos, $.Qt.ListWidget.new())
	me.listWidgetPhotos.setContextMenuPolicy($.Qt.CustomContextMenu)
	me.listWidgetPhotos.connect(#customContextMenuRequested, me, #slotPhotoContextMenuRequested)
	me.listWidgetPhotos.setFrameShape(0)
	me.listWidgetPhotos.setViewMode($.Qt.QListView.IconMode)
	me.listWidgetPhotos.setIconSize(150, 150)
	me.listWidgetPhotos.setResizeMode($.Qt.QListView.Adjust)
	me.listWidgetPhotos.setRawDropMode(TRUE)
	me.listWidgetPhotos.connect(#doubleClicked, me, #slotPhotoDoubleClicked)
	me.listWidgetPhotos.setAcceptDrops(TRUE)
	$.Qt.registerDropHandler(me.listWidgetPhotos, #slotPhotoDropped)
	me.tabWidget.addTab(me.listWidgetPhotos, "Photos in Set")
	me.tabWidget.setTabEnabled(2, FALSE)

	----------------------------------------
	-- Tab 3: Photo
	----------------------------------------
	me.setProp(#labelView, $.Qt.Label.new())
	me.labelView.setStyleSheet("QLabel{background:black;}")
	me.labelView.setScaledContents(TRUE)
	me.labelView.setContextMenuPolicy($.Qt.CustomContextMenu)
	me.labelView.connect(#customContextMenuRequested, me, #slotViewContextMenuRequested)
	me.setProp(#treeWidgetInfos, $.Qt.TreeWidget.new())
	me.treeWidgetInfos.setHeaderHidden(TRUE)
	splitter = $.Qt.Splitter.new()
	splitter.addWidget(me.labelView)
	splitter.addWidget(me.treeWidgetInfos)
	splitter.setSizes([800,200])
	splitter.setCollapsible(1, TRUE)
	me.tabWidget.addTab(splitter, "Photo")
	me.tabWidget.setTabEnabled(3, FALSE)

	me.resize(1024, 800)
	me.show()

	return me
end

----------------------------------------
-- Registers custom protocol "flickrauth:" and links it to this exe.
----------------------------------------
on registerProtocol (me)
	HKEY_CURRENT_USER = 1
	keyName = "Software\Classes\flickrauth"
	$.registry.createKey(keyName, HKEY_CURRENT_USER)
	$.registry.writeString(keyName, "", "URL:flickrauth", HKEY_CURRENT_USER)
	$.registry.writeString(keyName, "URL Protocol", "", HKEY_CURRENT_USER)
	p = QUOTE & _player.applicationPath & _player.applicationName & QUOTE
	$.registry.createKey(keyName&"\DefaultIcon", HKEY_CURRENT_USER)
	$.registry.writeString(keyName&"\DefaultIcon", "", p, HKEY_CURRENT_USER)
	$.registry.createKey(keyName&"\shell\open\command", HKEY_CURRENT_USER)
	$.registry.writeString(keyName&"\shell\open\command", "", p && QUOTE&"%1"&QUOTE, HKEY_CURRENT_USER)
end

----------------------------------------
-- Prints message to statusBar
----------------------------------------
on msg (me, msg)
	me.statusbar.showMessage(string(msg), 5000)
end

----------------------------------------
-- Prints error message to statusBar
----------------------------------------
on err (me, msg)
	me.statusbar.showMessage("Error: "&string(msg), 5000)
end

----------------------------------------
-- Loads all photo sets of account
----------------------------------------
on loadAccount (me)
	me.listWidgetPhotoSets.clear()

	me._current_user_id = $.flickr.get_user_id()
	if not $.filesystem.folderExists(me._cache_dir&"\"&me._current_user_id) then
		$.filesystem.folderCreate(me._cache_dir&"\"&me._current_user_id)
	end if

	-- get all sets
	photosets = []
	i = 1
	repeat while TRUE
		res = $.flickr.account_get_photosets(i)
		if res["stat"]<>"ok" then
			return err(res)
		end if
		repeat with photoset in res["photosets"]["photoset"]
			photosets.add(photoset)
		end repeat
		if i>=res["photosets"]["pages"] then
			exit repeat
		end if
		i = i + 1
	end repeat

	me._download_queue = []

	repeat with photoset in photosets
		listItem = $.Qt.ListWidgetItem.new()
		listItem.setText(photoset["title"]["_content"])
		listItem.setData($.Qt.UserRole, photoset["id"])
		iconFile = me._cache_dir & "\" & me._current_user_id & "\" & photoset["primary"] & ".jpg"
		if $.filesystem.fileExists(iconFile) then
			listItem.setIcon(iconFile)
		else
			me._download_queue.add([#listItem:listItem, #user_id:me._current_user_id, #photo_id:photoset["primary"], #iconFile:iconFile])
			listItem.setIcon(me._dummy_icon)
		end if
		me.listWidgetPhotoSets.addItem(listItem)
	end repeat

	-- special item: photos without set
	listItem = $.Qt.ListWidgetItem.new()
	listItem.setText("Photos not in set")
	listItem.setData($.Qt.UserRole, EMPTY)
	img = image(200,200,32)
	img.fill(img.rect, rgb(128,128,128))
	listItem.setIcon(img)
	me.listWidgetPhotoSets.addItem(listItem)

	me.tabWidget.setTabEnabled(1, TRUE)
	me.tabWidget.setCurrentIndex(1)

	me.msg(photosets.count&&"Sets in Account.")

	-- start the download queue
	me.slotSetThumbnailLoaded()
end

----------------------------------------
-- Loads all photos of photo set
----------------------------------------
on loadPhotoSet (me)
	me.listWidgetPhotos.clear()

	if me._current_set_id=EMPTY then
		root = "photos"
	else
		root = "photoset"
	end if

	-- get all photos
	photos = []
	i = 1
	repeat while TRUE
		res = $.flickr.photoset_get_photos(me._current_set_id, i)
		if res["stat"]<>"ok" then
			return err(res)
		end if
		repeat with p in res[root]["photo"]
			photos.add(p)
		end repeat
		if i>=res[root]["pages"] then
			exit repeat
		end if
		i = i + 1
	end repeat

	me._download_queue = []

	repeat with photo in photos
		listItem = $.Qt.ListWidgetItem.new()
		listItem.setText(photo["title"])
		listItem.setData($.Qt.UserRole, photo["id"])

		iconFile = me._cache_dir & "\" & me._current_user_id & "\" & photo["id"] & ".jpg"
		if $.filesystem.fileExists(iconFile) then
			listItem.setIcon(iconFile)
		else
			me._download_queue.add([#listItem:listItem, #set_id:me._current_set_id, #photo_id:photo["id"], #iconFile:iconFile])
			listItem.setIcon(me._dummy_icon)
		end if
		me.listWidgetPhotos.addItem(listItem)
	end repeat

	me.listWidgetPhotos.setCurrentRow(1)
	me.tabWidget.setTabEnabled(2, TRUE)
	me.tabWidget.setCurrentIndex(2)

	me.msg(photos.count&&"Photos in Set.")

	-- start the download queue
	me.slotPhotoThumbnailLoaded()
end

----------------------------------------
-- Loads photo
----------------------------------------
on loadPhoto (me, photo_id)
	res = $.flickr.photo_get_bytes(photo_id, "Original", #slotDownloadProgress)
	if res["stat"]="ok" then
		format = res["url"].char[res["url"].length-2..res["url"].length]
		me._qimg.loadData(res["data"], format)
		if not me._qimg.isNull() then
			me.labelView.setImage(me._qimg.save())
		end if
	end if
	-- get infos
	res = $.flickr.photo_get_info(photo_id)
	if res["stat"]="ok" then
		res.deleteProp("stat")
		me._current_photo = res["photo"]
		res = $.flickr.photo_get_exif(photo_id)
		if res["stat"]="ok" then
			me._current_photo["exif"] = [:]
			if not voidP(res["photo"]["camera"]) then
				me._current_photo["exif"]["Camera"] = res["photo"]["camera"]
			end if
			cnt = count(res["photo"]["exif"])
			repeat with i = 1 to cnt
				me._current_photo["exif"][ res["photo"]["exif"][i]["label"] ] = res["photo"]["exif"][i]["raw"]
			end repeat
		end if
		me.showInfos(me._current_photo)
	end if

	me.progressBar.setValue(0)
	me.tabWidget.setTabEnabled(3, TRUE)
	me.tabWidget.setCurrentIndex(3)
end

----------------------------------------
-- Shows metadata in QTreeWidget "treeWidgetInfos"
----------------------------------------
on showInfos (me, infos)
	rootItem = me.treeWidgetInfos.invisibleRootItem()
	me.clearItem(rootItem)
	me.propListToTreeItem(infos, rootItem, TRUE)
	me.treeWidgetInfos.sortbyColumn(0)
	cnt = rootItem.childCount()
	repeat with i = 1 to cnt
		rootItem.child(i-1).setExpanded(TRUE)
	end repeat
end

----------------------------------------
-- Recursively deletes a QWidgetTreeItem
----------------------------------------
on clearItem (me, treeWidgetItem)
	childItems = treeWidgetItem.takeChildren()
	repeat with childItem in childItems
		if childItem.childCount() then
			me.clearItem(childItem)
		end if
		childItem.destroy()
	end repeat
end

----------------------------------------
-- Helper function for displaying a nested propList
----------------------------------------
on propListToTreeItem (me, props, treeItem, isFileNode)
	childItems = []
	cnt = count(props)
	repeat with i = 1 to cnt
		prop = string(props.getPropAt(i))
		val = props[i]
		if integerP(val) then
			val = string(val)
		end if
		if ilk(val)=#propList then
			childItem = $.Qt.TreeWidgetItem.new()
			if not voidP(val["_content"]) then
				childItem.setText(0, prop&": "&val["_content"])
				childItem.setToolTip(0, "<div>"&val["_content"]&"</div>")
			else
				childItem.setText(0, prop)
				me.propListToTreeItem(val, childItem)
			end if
			childItems.add(childItem)
		else if ilk(val)=#list then
			repeat with v in val
				childItem = $.Qt.TreeWidgetItem.new()
				if not voidP(v["_content"]) then
					childItem.setText(0, prop&": "&v["_content"])
					childItem.setToolTip(0, "<div>"&v["_content"]&"</div>")
				else
					childItem.setText(0, prop)
					me.propListToTreeItem(v, childItem)
				end if
				childItems.add(childItem)
			end repeat
		else if ilk(val)=#string then
			if val="" then
				next repeat
			end if
			if ["lastupdate","posted","dateuploaded"].getPos(prop) then
				val = me.dateToDateTimeString(qx_dateFromUnixTimeStamp(integer(val)))
			end if
			childItem = $.Qt.TreeWidgetItem.new()
			childItem.setText(0, prop&": "&val)
			childItem.setToolTip(0, "<div>"&val&"</div>")
			childItems.add(childItem)
		end if
	end repeat
	treeItem.addChildren(childItems)
end

----------------------------------------
-- Returns string representation of given date object in YYYY-MM-DD HH:MM:II format.
----------------------------------------
on dateToDateTimeString (me, tDateObj)
	str = ""
	s = string(tDateObj.year)
	if s.length<4 then
		put "0000".char[1..4-s.length] before s
	end if
	put s after str
	s = string(tDateObj.month)
	if s.length<2 then
		s = "0"&s
	end if
	put s after str
	s = string(tDateObj.day)
	if s.length<2 then
		put "0" before s
	end if
	put s after str
	sec = tDateObj.seconds
	s = string(sec / 3600)
	sec = sec mod 3600
	if s.length<2 then
		put "0" before s
	end if
	put s after str
	s = string(sec / 60)
	sec = sec mod 60
	if s.length<2 then
		put "0" before s
	end if
	put s after str
	s = string(sec)
	if s.length<2 then
		put "0" before s
	end if
	put s after str
	put ":" after char 12 of str
	put ":" after char 10 of str
	put " " after char 8 of str
	put "-" after char 6 of str
	put "-" after char 4 of str
	return str
end

on __SLOTS
end

----------------------------------------
-- @callback
----------------------------------------
on slotShowConsoleToggled (me, sender, checked)
	$.console.consoleShow(checked)
	$.flickr.set_verbose(checked)
end

----------------------------------------
-- @callback
----------------------------------------
on slotAuthMessageReceived (me, args)
	str = args[5].readRawString(args[5].length)
	if not str starts "flickrauth:" then
		return
	end if
	delete char 1 to 12 of str -- "flickrauth:?"
	od = _player.itemDelimiter
	_player.itemDelimiter = "&"
	cnt = str.item.count
	repeat with i = 1 to cnt
		if str.item[i] starts "oauth_verifier=" then
			verifier = str.item[i].char[16..str.item[i].length]
			exit repeat
		end if
	end repeat
	if voidP(verifier) then
		return
	end if
	res = $.flickr.get_access_token(verifier)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	user = $.flickr.login(res["access_token"], res["access_token_secret"])
	if user["stat"]<>"ok" then
		return me.err(user)
	end if
	account_name = user["user"]["username"]["_content"]

	-- save in accounts.ini
	me._account_settings.beginGroup(account_name)
	me._account_settings.setValue("access_token", res["access_token"])
	me._account_settings.setValue("access_token_secret", res["access_token_secret"])
	me._account_settings.endGroup()

	-- add to list
	listItem = $.Qt.ListWidgetItem.new()
	listItem.setText(account_name)
	listItem.setData($.Qt.UserRole, res["access_token"])
	listItem.setData($.Qt.UserRole+1, res["access_token_secret"])
	me.listWidgetAccounts.addItem(listItem)
	me.listWidgetAccounts.setCurrentItem(listItem)
end

----------------------------------------
-- @callback
----------------------------------------
on slotAccountNew (me)
	res = $.flickr.get_request_token("flickrauth:")
	if res["stat"]<>"ok" then
		me.err(res)
	end if
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoSetDoubleClicked (me, sender, row)
	listItem = sender.item_(row)
	me._current_set_id = listItem.data($.Qt.userRole)
	me.loadPhotoSet()
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoSetLoad (me, sender)
	listItem = me.listWidgetPhotoSets.item_(me.listWidgetPhotoSets.currentRow())
	me._current_set_id = listItem.data($.Qt.userRole)
	me.loadPhotoSet()
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoSetDownload (me, sender)
	listItem = me.listWidgetPhotoSets.item_(me.listWidgetPhotoSets.currentRow())
	me._current_set_id = listItem.data($.Qt.userRole)

	--loadPhotoSet()
	if me._current_set_id=EMPTY then
		root = "photos"
	else
		root = "photoset"
	end if

	me._download_queue = []
	i = 1
	repeat while TRUE
		res = $.flickr.photoset_get_photos(me._current_set_id, i)
		if res["stat"]<>"ok" then
			return me.err(res)
		end if
		repeat with photo in res[root]["photo"]
			me._download_queue.add(photo["id"])
		end repeat
		if i>=res[root]["pages"] then
			exit repeat
		end if
		i = i + 1
	end repeat

	me._download_queue_len = me._download_queue.count
	if me._download_queue_len=0 then
		return -- should be impossible
	end if
	me.slotPhotoDownloaded()
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoDownloaded (me, res, ch, custom_data)
	pct = (me._download_queue_len-me._download_queue.count)*100/me._download_queue_len
	me.progressBar.setValue(pct)
	if me._download_queue.count = 0 then
		me.msg("All Photos successfully downloaded.")
		me.progressBar.setValue(0)
		return
	end if
	photo_id = me._download_queue[1]
	me._download_queue.deleteAt(1)
	$.flickr.photo_download_original(photo_id, me._downloads_dir, VOID,VOID, #slotPhotoDownloaded)
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoDoubleClicked (me, sender, row)
	listItem = sender.item_(row)
	photo_id = listItem.data($.Qt.userRole)
	me.loadPhoto(photo_id)
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoLoad (me, sender)
	listItem = me.listWidgetPhotos.item_(me.listWidgetPhotos.currentRow())
	photo_id = listItem.data($.Qt.userRole)
	me.loadPhoto(photo_id)
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoDownload (me, sender)
	listItem = me.listWidgetPhotos.item_(me.listWidgetPhotos.currentRow())
	photo_id = listItem.data($.Qt.userRole)
	res = $.flickr.photo_download_original(photo_id, me._downloads_dir)
	if res["stat"]<>"ok" then
		me.err(res)
	end if
end

--------------------------------------
-- @callback
--------------------------------------
on slotAbout (me)
	msg = "<b>MyFlickrManager v0.11</b><br>(c) 2020 Valentin Schmidt<br><br>"&\
	"A simple Flickr Photo Manager based on Lingo, Qt and Curl"
	$.Qt.MessageBox.about(me, "About MyFlickrManager", msg)
end

--------------------------------------
-- @callback
--------------------------------------
on slotAccountContextMenuRequested (me, sender, p)
	row = me.listWidgetAccounts.currentRow()
	if row<0 then
		return
	end if
	m = $.Qt.Menu.new(me)
	action = $.Qt.Action.new(m)
	action.setText("Load Account")
	action.connect(#triggered, me, #slotAccountLoad)
	m.addAction(action)
	m.addSeparator()
	action = $.Qt.Action.new(m)
	action.setText("Rename Account")
	action.connect(#triggered, me, #slotAccountRename)
	m.addAction(action)
	action = $.Qt.Action.new(m)
	action.setText("Remove Account")
	action.connect(#triggered, me, #slotAccountRemove)
	m.addAction(action)
	m.addSeparator()
	action = $.Qt.Action.new(m)
	action.setText("Clear Cache")
	action.connect(#triggered, me, #slotAccountClearCache)
	m.addAction(action)
	m.exec(p)
end

--------------------------------------
-- @callback
--------------------------------------
on slotAccountRename (me)
	row = me.listWidgetAccounts.currentRow()
	listItem = me.listWidgetAccounts.item_(row)
	account_name = listItem.text()
	new_name = $.Qt.InputDialog.getText(me, "Rename Acount", "New Name:", 0, account_name)
	if string(new_name)="" or new_name=account_name then
		return
	end if
	-- update in accounts.ini
	me._account_settings.beginGroup(new_name)
	me._account_settings.setValue("access_token", listItem.data($.Qt.UserRole))
	me._account_settings.setValue("access_token_secret", listItem.data($.Qt.UserRole+1))
	me._account_settings.endGroup()
	me._account_settings.remove(account_name)
	-- update item
	listItem.setText(new_name)
end

--------------------------------------
-- @callback
--------------------------------------
on slotAccountRemove (me)
	row = me.listWidgetAccounts.currentRow()
	listItem = me.listWidgetAccounts.item_(row)
	account_name = listItem.text()
	me._account_settings.remove(account_name)
	me.listWidgetAccounts.removeItem(row)
end

--------------------------------------
-- @callback
--------------------------------------
on slotAccountClearCache (me)
	listItem = me.listWidgetAccounts.item_(me.listWidgetAccounts.currentRow())
	access_token = listItem.data($.Qt.UserRole)
	access_token_secret = listItem.data($.Qt.UserRole+1)
	$.flickr.login(access_token, access_token_secret)
	dir = me._cache_dir & "\" & $.flickr.get_user_id()
	files = $.filesystem.folderList(dir)
	repeat with f in files
		$.filesystem.fileDelete(dir&"\"&f)
	end repeat
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotoSetContextMenuRequested (me, sender, p)
	row = me.listWidgetPhotoSets.currentRow()
	if row<0 then
		return
	end if

	listItem = me.listWidgetPhotoSets.item_(row)
	photoset_id = listItem.data($.Qt.UserRole)
	if photoset_id=EMPTY then
		return
	end if

	m = $.Qt.Menu.new(me)

	action = $.Qt.Action.new(m)
	action.setText("Load Photo Set")
	action.connect(#triggered, me, #slotPhotoSetLoad)
	m.addAction(action)

	m.addSeparator()

	action = $.Qt.Action.new(m)
	action.setText("Download Photo Set (Originals)")
	action.connect(#triggered, me, #slotPhotoSetDownload)
	m.addAction(action)

	m.addSeparator()

	m2 = m.addMenu("Copy *all* Photos of Set to other Set")
	cnt = me.listWidgetPhotoSets.count_()
	repeat with i = 1 to cnt
		listItem = me.listWidgetPhotoSets.item_(i-1)
		set_id = listItem.data($.Qt.UserRole)
		if set_id=photoset_id or set_id=EMPTY then
			next repeat
		end if
		action = $.Qt.Action.new(m2)
		action.setText(listItem.text())
		action.setData(set_id)
		action.connect(#triggered, me, #slotPhotosCopyAll)
		m2.addAction(action)
	end repeat

	m2 = m.addMenu("Move *all* Photos of Set to other Set")
	cnt = me.listWidgetPhotoSets.count_()
	repeat with i = 1 to cnt
		listItem = me.listWidgetPhotoSets.item_(i-1)
		set_id = listItem.data($.Qt.UserRole)
		if set_id=photoset_id or set_id=EMPTY then
			next repeat
		end if
		action = $.Qt.Action.new(m2)
		action.setText(listItem.text())
		action.setData(set_id)
		action.connect(#triggered, me, #slotPhotosMoveAll)
		m2.addAction(action)
	end repeat

	m.addSeparator()

	action = $.Qt.Action.new(m)
	action.setText("Rename Photo Set")
	action.connect(#triggered, me, #slotPhotoSetRename)
	m.addAction(action)

	action = $.Qt.Action.new(m)
	action.setText("Delete Photo Set")
	action.connect(#triggered, me, #slotPhotoSetDelete)
	m.addAction(action)

	m.exec(p)
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotoSetDelete (me)
	row = me.listWidgetPhotoSets.currentRow()
	if row<0 then
		return
	end if
	listItem = me.listWidgetPhotoSets.item_(row)
	photoset_id = listItem.data($.Qt.UserRole)
	if photoset_id=EMPTY then
		return
	end if
	res = $.flickr.photoset_delete(photoset_id)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	me.listWidgetPhotoSets.removeItem(row)
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotoContextMenuRequested (me, sender, p)
	row = me.listWidgetPhotos.currentRow()
	if row<0 then
		return
	end if

	m = $.Qt.Menu.new(me)

	action = $.Qt.Action.new(m)
	action.setText("Load Photo")
	action.connect(#triggered, me, #slotPhotoLoad)
	m.addAction(action)

	action = $.Qt.Action.new(m)
	action.setText("Download Photo")
	action.connect(#triggered, me, #slotPhotoDownload)
	m.addAction(action)

	m.addSeparator()

	m2 = m.addMenu("Copy Photo to Set")
	cnt = me.listWidgetPhotoSets.count_()
	repeat with i = 1 to cnt
		listItem = me.listWidgetPhotoSets.item_(i-1)
		set_id = listItem.data($.Qt.UserRole)
		if set_id=me._current_set_id or set_id=EMPTY then
			next repeat
		end if
		action = $.Qt.Action.new(m2)
		action.setText(listItem.text())
		action.setData(set_id)
		action.connect(#triggered, me, #slotPhotoCopy)
		m2.addAction(action)
	end repeat

	m2 = m.addMenu("Move Photo to Set")
	cnt = me.listWidgetPhotoSets.count_()
	repeat with i = 1 to cnt
		listItem = me.listWidgetPhotoSets.item_(i-1)
		set_id = listItem.data($.Qt.UserRole)
		if set_id=me._current_set_id or set_id=EMPTY then
			next repeat
		end if
		action = $.Qt.Action.new(m2)
		action.setText(listItem.text())
		action.setData(set_id)
		action.connect(#triggered, me, #slotPhotoMove)
		m2.addAction(action)
	end repeat

	m.addSeparator()

	if me._current_set_id<>EMPTY then
		action = $.Qt.Action.new(m)
		action.setText("Set this Photo as primary for Set")
		action.connect(#triggered, me, #slotPhotoSetPrimary)
		m.addAction(action)

		action = $.Qt.Action.new(m)
		action.setText("Remove Photo from Set")
		action.connect(#triggered, me, #slotPhotoRemove)
		m.addAction(action)
	else
		action = $.Qt.Action.new(m)
		action.setText("Create new Set with this Photo as primary")
		action.connect(#triggered, me, #slotPhotoNewSet)
		m.addAction(action)
	end if

	action = $.Qt.Action.new(m)
	action.setText("Rename Photo")
	action.connect(#triggered, me, #slotPhotoRename)
	m.addAction(action)

	action = $.Qt.Action.new(m)
	action.setText("Delete Photo")
	action.connect(#triggered, me, #slotPhotoDelete)
	m.addAction(action)

	m.exec(p)
end

--------------------------------------
-- @callback
--------------------------------------
on slotViewContextMenuRequested (me, sender, p)
	m = $.Qt.Menu.new(me)
	action = $.Qt.Action.new(m)
	action.setText("Edit Description")
	action.connect(#triggered, me, #slotPhotoEditDescription)
	m.addAction(action)
	action = $.Qt.Action.new(m)
	action.setText("Add Tags")
	action.connect(#triggered, me, #slotPhotoAddTags)
	m.addAction(action)
	if count(me._current_photo["tags"]["tag"]) then
		m2 = m.addMenu("Remove Tag")
		repeat with tag in me._current_photo["tags"]["tag"]
			action = $.Qt.Action.new(m2)
			action.setText(tag["_content"])
			action.setData(tag["id"])
			action.connect(#triggered, me, #slotPhotoRemoveTag)
			m2.addAction(action)
		end repeat
	end if
	m.exec(p)
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotoCopy (me, sender)
	row = me.listWidgetPhotos.currentRow()
	photo_id = me.listWidgetPhotos.item_(row).data($.Qt.UserRole)
	target_set_id = sender.data()
	res = $.flickr.photoset_add_photo(target_set_id, photo_id)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	me.msg("Photo successfully copied.")
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotoMove (me, sender)
	row = me.listWidgetPhotos.currentRow()
	photo_id = me.listWidgetPhotos.item_(row).data($.Qt.UserRole)
	target_set_id = sender.data()
	res = $.flickr.photoset_add_photo(target_set_id, photo_id)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	if me._current_set_id<>EMPTY then
		res = $.flickr.photoset_remove_photo(me._current_set_id, photo_id)
		if res["stat"]<>"ok" then
			return me.err(res)
		end if
	end if
	me.listWidgetPhotos.removeItem(row)
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotoEditDescription (me, sender)
	desc_old = me._current_photo["description"]["_content"]
	desc = $.Qt.InputDialog.getText(me, "Edit Description", "New description:", 0, desc_old)
	if string(desc)="" or desc=desc_old then
		return
	end if
	photo_id = me._current_photo["id"]
	res = $.flickr.photo_set_description(photo_id, desc)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	-- reload infos
	res = $.flickr.photo_get_info(photo_id)
	if res["stat"]="ok" then
		res.deleteProp("stat")
		me._current_photo = res["photo"]
		showInfos(me._current_photo)
	end if
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotoAddTags (me, sender)
	tags = $.Qt.InputDialog.getText(me, "Add Tags", "Tags separated by comma:", 0, "")
	if string(tags)="" then
		return
	end if
	photo_id = me._current_photo["id"]
	res = $.flickr.photo_add_tags(photo_id, tags)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	-- reload infos
	res = $.flickr.photo_get_info(photo_id)
	if res["stat"]="ok" then
		res.deleteProp("stat")
		me._current_photo = res["photo"]
		showInfos(me._current_photo)
	end if
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotoRemoveTag (me, sender)
	photo_id = me._current_photo["id"]
	tag_id = sender.data()
	res = $.flickr.photo_remove_tag(photo_id, tag_id)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	-- reload infos
	res = $.flickr.photo_get_info(photo_id)
	if res["stat"]="ok" then
		res.deleteProp("stat")
		me._current_photo = res["photo"]
		me.showInfos(me._current_photo)
	end if
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotosCopyAll (me, sender)
	row = me.listWidgetPhotoSets.currentRow()
	listItem = me.listWidgetPhotoSets.item_(row)
	source_set_id = listItem.data($.Qt.UserRole)
	target_set_id = sender.data()

	-- get all photos of source set
	source_photo_ids = []
	i = 1
	repeat while TRUE
		res = $.flickr.photoset_get_photos(source_set_id, i)
		if res["stat"]<>"ok" then
			return me.err(res)
		end if
		repeat with photo in res["photoset"]["photo"]
			source_photo_ids.add(photo["id"])
		end repeat
		if i>=res["photoset"]["pages"] then
			exit repeat
		end if
		i = i + 1
	end repeat

	-- start copying
	me.tabWidget.setDisabled(True)
	cnt = source_photo_ids.count
	repeat with i = 1 to cnt
		photo_id = source_photo_ids[i]
		res = $.flickr.photoset_add_photo(target_set_id, photo_id)
		if res["stat"]<>"ok" then
			exit repeat
		end if
		me.progressBar.setValue(i*100/cnt)
		me.progressBar.repaint()
	end repeat
	me.tabWidget.setDisabled(False)
	if i<cnt then
		me.err(res)
	else
		me.msg(cnt && "Photos copied successfully.")
	end if
	me.progressBar.setValue(0)
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotosMoveAll (me, sender)
	row = me.listWidgetPhotoSets.currentRow()
	listItem = me.listWidgetPhotoSets.item_(row)
	source_set_id = listItem.data($.Qt.UserRole)
	target_set_id = sender.data()

	-- get all photos of source set
	source_photo_ids = []
	i = 1
	repeat while TRUE
		res = $.flickr.photoset_get_photos(source_set_id, i)
		if res["stat"]<>"ok" then
			return me.err(res)
		end if
		repeat with photo in res["photoset"]["photo"]
			source_photo_ids.add(photo["id"])
		end repeat
		if i>=res["photoset"]["pages"] then
			exit repeat
		end if
		i = i + 1
	end repeat

	-- start moving
	me.tabWidget.setDisabled(True)
	cnt = source_photo_ids.count
	repeat with i = 1 to cnt
		photo_id = source_photo_ids[i]
		res = $.flickr.photoset_add_photo(target_set_id, photo_id)
		if res["stat"]<>"ok" then
			exit repeat
		end if
		res = $.flickr.photoset_remove_photo(source_set_id, photo_id)
		if res["stat"]<>"ok" then
			exit repeat
		end if
		me.progressBar.setValue(i*100/cnt)
		me.progressBar.repaint()
	end repeat
	me.tabWidget.setDisabled(False)
	if i<cnt then
		me.err(res)
	else
		me.msg(cnt && "Photos moved successfully.")
	end if
	me.progressBar.setValue(0)

	-- remove deleted set from the list
	me.listWidgetPhotoSets.removeItem(row)
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotoRemove (me, sender)
	row = me.listWidgetPhotos.currentRow()
	photo_id = me.listWidgetPhotos.item_(row).data($.Qt.UserRole)
	res = $.flickr.photoset_remove_photo(me._current_set_id, photo_id)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	me.listWidgetPhotos.removeItem(row)
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotoNewSet (me, sender)
	row = me.listWidgetPhotos.currentRow()
	primary_photo_id = me.listWidgetPhotos.item_(row).data($.Qt.UserRole)
	photoset_title = $.Qt.InputDialog.getText(me, "New Photoset", "Title of Photoset:", 0, "New Set")
	if string(photoset_title)="" then
		return
	end if
	res = $.flickr.photoset_create(primary_photo_id, photoset_title)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	me.loadAccount()
end

--------------------------------------
-- @callback
--------------------------------------
on slotPhotoDelete (me, sender)
	row = me.listWidgetPhotos.currentRow()
	photo_id = me.listWidgetPhotos.item_(row).data($.Qt.UserRole)
	res = $.flickr.photo_delete(photo_id)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	me.listWidgetPhotos.removeItem(row)
end

--------------------------------------
-- @callback
--------------------------------------
on slotAccountDoubleClicked (me, sender, row)
	listItem = me.listWidgetAccounts.item_(row)
	access_token = listItem.data($.Qt.UserRole)
	access_token_secret = listItem.data($.Qt.UserRole+1)
	res = $.flickr.login(access_token, access_token_secret)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	me.loadAccount()
end

--------------------------------------
-- @callback
--------------------------------------
on slotAccountLoad (me, sender)
	listItem = me.listWidgetAccounts.item_(me.listWidgetAccounts.currentRow())
	access_token = listItem.data($.Qt.UserRole)
	access_token_secret = listItem.data($.Qt.UserRole+1)
	$.flickr.login(access_token, access_token_secret)
	me.loadAccount()
end

----------------------------------------
-- @callback
----------------------------------------
on slotUploadProgress (me, dltotal, dlnow, ultotal, ulnow)
	if ultotal>0 then
		me.progressBar.setValue(ulnow*100/ultotal)
	end if
end

----------------------------------------
-- @callback
----------------------------------------
on slotDownloadProgress (me, dltotal, dlnow, ultotal, ulnow)
	if dltotal>0 then
		me.progressBar.setValue(dlnow*100/dltotal)
	end if
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoSetPrimary (me, sender)
	row = me.listWidgetPhotos.currentRow()
	photo_id = me.listWidgetPhotos.item_(row).data($.Qt.UserRole)
	res = $.flickr.photoset_set_primary_photo(me._current_set_id, photo_id)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	me.listWidgetPhotos.removeItem(row)
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoRename (me, sender)
	row = me.listWidgetPhotos.currentRow()
	if row<0 then
		return
	end if
	listItem = me.listWidgetPhotos.item_(row)
	old_title = listItem.text()
	photo_title = $.Qt.InputDialog.getText(me, "Rename Photo", "Title of Photo:", 0, old_title)
	if string(photo_title)="" or photo_title=old_title then
		return
	end if
	photo_id = me.listWidgetPhotos.item_(row).data($.Qt.UserRole)
	res = $.flickr.photo_set_title(listItem.data($.Qt.UserRole), photo_title)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	listItem.setText(photo_title)
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoSetRename (me, sender)
	row = me.listWidgetPhotoSets.currentRow()
	if row<0 then
		return
	end if
	listItem = me.listWidgetPhotoSets.item_(row)
	old_title = listItem.text()
	photoset_title = $.Qt.InputDialog.getText(me, "Rename PhotoSet", "Title of Photo:", 0, old_title)
	if string(photoset_title)="" or photoset_title=old_title then
		return
	end if
	photoset_id = me.listWidgetPhotoSets.item_(row).data($.Qt.UserRole)
	res = $.flickr.photoset_set_title(listItem.data($.Qt.UserRole), photoset_title)
	if res["stat"]<>"ok" then
		return me.err(res)
	end if
	listItem.setText(photoset_title)
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoSetDropped (me, args)
	row = me.listWidgetPhotoSets.currentRow()
	photoset_id = me.listWidgetPhotoSets.item_(row).data($.Qt.UserRole)
	if photoset_id=EMPTY then
		return
	end if

	p = args[2]
	targetItem = me.listWidgetPhotoSets.itemAt(p[1], p[2])
	targetRow = me.listWidgetPhotoSets.row(targetItem)

	movedItem = me.listWidgetPhotoSets.takeItem(row)
	me.listWidgetPhotoSets.insertItem(targetRow, movedItem)
	me.listWidgetPhotos.setCurrentItem(movedItem)

	-- get new order of ids
	photoset_ids = []
	cnt = me.listWidgetPhotoSets.count_()
	repeat with i = 1 to cnt
		listItem = me.listWidgetPhotoSets.item_(i-1)
		photoset_id = listItem.data($.Qt.UserRole)
		if photoset_id=EMPTY then
			next repeat
		end if
		photoset_ids.add(photoset_id)
	end repeat

	res = $.flickr.account_order_photosets(photoset_ids)
	if res["stat"]<>"ok" then
		me.err(res)
	end if
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoDropped (me, args)
	drop_data = args[1]
	if ilk(drop_data)=#list then -- file(s) dropped
		me._upload_queue = []
		repeat with fn in drop_data
			if ["gif","jpg","jpeg","png", "bmp","psd","tif","tiff"].getPos($.file.getFileType(fn)) then
				me._upload_queue.add([#set_id:me._current_set_id, #file:fn])
			end if
		end repeat
		if me._upload_queue.count>0 then
			me.slotUploadFinished()
		end if
	else if ilk(drop_data)=#propList then -- item moved
		p = args[2]
		targetItem = me.listWidgetPhotos.itemAt(p[1], p[2])
		targetRow = me.listWidgetPhotos.row(targetItem)
		movedItem = me.listWidgetPhotos.takeItem(me.listWidgetPhotos.currentRow())
		me.listWidgetPhotos.insertItem(targetRow, movedItem)
		me.listWidgetPhotos.setCurrentItem(movedItem)
		-- get new order of ids
		photo_ids = []
		cnt = me.listWidgetPhotos.count_()
		repeat with i = 1 to cnt
			listItem = me.listWidgetPhotos.item_(i-1)
			photo_id = listItem.data($.Qt.UserRole)
			photo_ids.add(photo_id)
		end repeat
		res = $.flickr.photoset_reorder_photos(me._current_set_id, photo_ids)
		if res["stat"]<>"ok" then
			me.err(res)
		end if
	end if
end

----------------------------------------
-- @callback
----------------------------------------
on slotUploadFinished (me, res)
	me.progressBar.setValue(0)
	if not voidP(res) then
		if res["stat"]<>"ok" then
			me.err(res)
		end if
	end if
	if me._upload_queue.count = 0 then
		return me.loadPhotoSet()
	end if
	data = me._upload_queue[1]
	me._upload_queue.deleteAt(1)
	$.flickr.photo_upload(data.set_id, data.file, #slotUploadProgress, me, #slotUploadFinished)
end

----------------------------------------
-- @callback
----------------------------------------
on slotSetThumbnailLoaded (me, res, ch, custom_data)
	if not voidP(res) then
		if custom_data.user_id<>me._current_user_id then
			return
		end if
		custom_data.listItem.setIcon(custom_data.iconFile)
	end if
	if me._download_queue.count = 0 then
		return
	end if
	download_data = me._download_queue[1]
	me._download_queue.deleteAt(1)
	$.flickr.photo_download(download_data.photo_id, "Large Square", download_data.iconFile, VOID,VOID, #slotSetThumbnailLoaded, me, download_data)
end

----------------------------------------
-- @callback
----------------------------------------
on slotPhotoThumbnailLoaded (me, res, ch, custom_data)
	if not voidP(res) then
		if custom_data.set_id<>me._current_set_id then
			return
		end if
		custom_data.listItem.setIcon(custom_data.iconFile)
	end if
	if me._download_queue.count = 0 then
		return
	end if
	download_data = me._download_queue[1]
	me._download_queue.deleteAt(1)
	$.flickr.photo_download(download_data.photo_id, "Large Square", download_data.iconFile, VOID,VOID, #slotPhotoThumbnailLoaded, me, download_data)
end
